%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% Funktion        : Kollisioncheck Kanten<-->Ebenen                       %
%                                                                         %
%                                                                         %
% Eingabe         : Kantenvektoren und Ebenen eines Kollisionsobjektes    %
%                                                                         %
%                   edge: Kantenvektor                                    %
%                   nx6 Vektor --> n:Anzahl der Kanten                    %
%                   6:[x1 y1 z1 x2 y2 z2] bestehend aus 2 Punkten         %
%                                                                         %
%                   plane: Ebenen des Kollisionsobjektes                  %
%                   1x9x6 Matrix bestehend aus 6 Ebenen, welche das Quader%
%                   reprsentieren                                        %
%                   1x9: [X0 Y0 Z0  DX1 DY1 DZ1  DX2 DY2 DZ2]             %
%                   (X0, Y0, Z0) ein Punkt welcher auf der Ebene liegt    %
%                   (DX1, DY1, DZ1) erster Richtungsvektor                %
%                   (DX2, DY2, DZ2) zweiter Richtungsvektor               %
%                   Die zwei Richtungsvektoren sind otghogonal &          %
%                   normalisiert                                          %
%                                                                         %
%                   range: Min/Max Werte von x,y,z                        %
%                   3x2 Matrix: [xmin xmax ; ymin ymax ; zmin zmax]       %
%                                                                         %
%                                                                         %
% Ausgabe         : collision: Kollisonsvektor nx1 n: Kollisioncheck      %
%                   fr jede Kante --> 1:true ; 0:false                   %
%                                                                         %
% Autor           : Kevin Penner                                          %
%                                                                         %
% Implementation  : MATLAB R2016b                                         %
%                                                                         %
% Bemerkung       :                                                       %
%                                                                         %
% nderungsdatum  : 27.04.2017                                            %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [collision] = CollisonEdgePlane(edge, plane, range)
    
    %Anzahl der Kanten aus Vektor lesen
    [nedges,~] = size(edge);
    
    %Kollisionsvektor erzeugen
    collision = zeros(nedges,1);
    
    %Schnittpunkte fr jede Kante bestimmen
    for e=1:nedges
        %Schnittpunkt der Kante zu allen Ebenen (Flchen) berechnen
        for p=1:6
            %p:Ebene (1 bis 6) - e:Kante (1 bis nedges) - : --> [x y z]
            %Koordinate des Schnittpunktes
            %Beispiel: intersection(1,:,2) Schnittpunkt der 1 Ebene mit
            %Kante 2
            intersection(p,:,e) = intersectEdgePlane(edge(e,:), plane(:,:,p));
            
            %Kollision berprfen, wenn eine vorliegt
            if sum(isnan(intersection(p,:,e))) == 0
                
                %berprfen, ob Schnittpunkt im Kollisionbereich der Ebene liegt
                if intersection(p,1,e) >= range(1,1) && intersection(p,1,e) <= range(1,2)...
                   && intersection(p,2,e) >= range(2,1) && intersection(p,2,e) <= range(2,2)...
                   && intersection(p,3,e) >= range(3,1) && intersection(p,3,e) <= range(3,2)
                    
                    %Wenn Schnittpunkt im Kollisionsbereich --> Flag setzen
                    %und Abbruch fr die Kollisonsberprfung fr diese
                    %Kante --> nchste Kante prfen
                    collision(e,1) = 1;
                    break;
                end
     
            end 
            
        end
       
    end
  
end