%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% Funktion        : Erzeugen eines Kollisionsobjektes im Raum des         %
%                   Roboters                                              %
%                                                                         %
%                                                                         %
% Eingabe         : Ortvektor und 3 Richtungsvektoren, welche ein Quader  %
%                   bzw. Parallelogramm erzeugen                          %
%                                                                         %
%                   o,u,v,w: 1x3 Vektoren                                 %
%                   o: Ortsvektor zu einer Ecke                           %
%                   u: Richtungsvektor (Breite)                           %
%                   v: Richtungsvektor (Hhe)                             %
%                   w: Richtungsvektor (Tiefe)                            %
%                   offset: offset fr min/max Erzeugung                  %
%                   --> wie nah zhlt als "geschnitten"                   %
%                                                                         %
% Ausgabe         : box: Eckverbindungen fr Plot                         %
%                   nx1 Vektor mit n Eckpunkten (Haus vom Nikolaus        %
%                   Prinzip)                                              %
%                                                                         %
%                   plane: Ebenen des Quaders                             %
%                   1x9x6 Matrix bestehend aus 6 Ebenen, welche das Quader%
%                   reprsentieren                                        %
%                   1x9: [X0 Y0 Z0  DX1 DY1 DZ1  DX2 DY2 DZ2]             %
%                   (X0, Y0, Z0) ein Punkt welcher auf der Ebene liegt    %
%                   (DX1, DY1, DZ1) erster Richtungsvektor                %
%                   (DX2, DY2, DZ2) zweiter Richtungsvektor               %
%                   Die zwei Richtungsvektoren sind otghogonal &          %
%                   normalisiert                                          %
%                                                                         %
%                   range: Min/Max Werte von x,y,z                        %
%                   3x2 Matrix: [xmin xmax ; ymin ymax ; zmin zmax]       %
%                                                                         %
% Autor           : Kevin Penner                                          %
%                                                                         %
% Implementation  : MATLAB R2016b                                         %
%                                                                         %
% Bemerkung       :                                                       %
%                                                                         %
% nderungsdatum  : 27.04.2017                                            %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [box,plane,range] = CreateKollisionObject(o,u,v,w,offset)
    
    %Eckpunkte erstellen
    corners = [o;...
               o+u;...
               o+u+v;...
               o+v;...
               o+v+w;...
               o+w;...
               o+w+u;...
               o+w+u+v;...
               ];
           
    %Eckenverbindungen erstellen --> nur fr Darstellung
    box = [corners(1,:);...
           corners(2,:);...
           corners(3,:);...
           corners(4,:);...
           corners(5,:);...
           corners(6,:);...
           corners(1,:);...
           corners(6,:);...
           corners(7,:);...
           corners(8,:);...
           corners(5,:);...
           corners(6,:);...
           corners(7,:);...
           corners(2,:);...
           corners(1,:);...
           corners(4,:);...
           corners(3,:);...
           corners(8,:);...
           corners(3,:);...
           corners(4,:);...
           corners(1,:);...
           ];
       
    %Berechnung der 6 Ebenen (Flchen) des Quaders
    plane(:,:,1) = createPlane([corners(1,:) ; corners(2,:) ; corners(3,:)]);
    plane(:,:,2) = createPlane([corners(2,:) ; corners(3,:) ; corners(8,:)]);
    plane(:,:,3) = createPlane([corners(8,:) ; corners(5,:) ; corners(6,:)]);
    plane(:,:,4) = createPlane([corners(6,:) ; corners(5,:) ; corners(4,:)]);
    plane(:,:,5) = createPlane([corners(1,:) ; corners(2,:) ; corners(7,:)]);
    plane(:,:,6) = createPlane([corners(4,:) ; corners(3,:) ; corners(8,:)]);
    
    %Extremwerte bestimmen
    range(1,1) = min(corners(:,1)) - offset;
    range(1,2) = max(corners(:,1)) + offset;
    range(2,1) = min(corners(:,2)) - offset;
    range(2,2) = max(corners(:,2)) + offset;
    range(3,1) = min(corners(:,3)) - offset;
    range(3,2) = max(corners(:,3)) + offset;
       
       
end