%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% Funktion        : Mit jedem Aufruf wird ein rekursives Kalman-Filter    %
%                   ausgefhrt.                                           %
%                                                                         %
%                                                                         %
% Eingabe         : PosMess: Aktueller Messvektor (1x3)                   %
%                   z.B. [x;y;z] --> Koordinaten                          %
%                                                                         %
%                   Q: Kovarianzmatrix des Systemrauschens                %
%                   (6x6) oder (9x9)                                      %
%                   Model = 'CONST_VELOCITY' --> (6x6)                    %
%                   Model = 'CONST_ACCELERATION' --> (9x9)                %
%                                                                         %
%                   R: Kovarianzmatrix des Messrauschens (3x3)            %
%                                                                         %
%                   x_old: Zustandsvektor aus vorherigem Zyklus           %
%                   bei FirstRun = 1 --> x_old = Messwerte                %
%                   Model = 'CONST_VELOCITY' --> x = [x;y;z;vx;vy:vz]     %
%                   Model = 'CONST_ACCELERATION' -->                      %
%                   x = [x;y;z;vx;vy:vz;ax;ay;az]                         %
%                                                                         %
%                   P_old: Kovarianzmatrix des Schtzfehlers aus          %
%                   vorherigem Zyklus                                     %
%                   bei FirstRun = 1 --> P_old = hoch                     %
%                   Model = 'CONST_VELOCITY' --> (6x6)                    %
%                   Model = 'CONST_ACCELERATION' --> (9x9)                %
%                                                                         %
%                   FirstRun: Erster Messwerte? Ja=1 ; Nein=0             %
%                                                                         %
%                   Model: Art des Bewegungsmodells                       %
%                   Model = 'CONST_VELOCITY'                              %
%                   Model = 'CONST_ACCELERATION'                          %
%                                                                         %
%                   dt: Zeitdifferenz zwischen den Messdaten              %
%                                                                         %
%                                                                         %
% Ausgabe         : x: aktueller Zustandsvektor                           %
%                   Model = 'CONST_VELOCITY' --> x = [x;y;z;vx;vy:vz]     %
%                   Model = 'CONST_ACCELERATION' -->                      %
%                   x = [x;y;z;vx;vy:vz;ax;ay;az]                         %
%                                                                         %
%                   P: aktuelle Kovarianzmatrix des Schtzfehlers         %
%                   Model = 'CONST_VELOCITY' --> (6x6)                    %
%                   Model = 'CONST_ACCELERATION' --> (9x9)                %
%                                                                         %
%                                                                         %
% Autor           : Kevin Penner                                          %
%                                                                         %
% Implementation  : MATLAB R2016b                                         %
%                                                                         %
% Bemerkung       :                                                       %
%                                                                         %
% nderungsdatum  : 03.05.2017                                            %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [x,P] = KalmanFilter(PosMess,Q,R,x_old,P_old,firstRun,Model,dt)

persistent H 

%% einmalige Initialisierung

if firstRun == 1
  
  if(strcmp('CONST_VELOCITY',Model))
        
  %Messmatrix
  H = [1 0 0 0 0 0;
       0 1 0 0 0 0;
       0 0 1 0 0 0];
  
  %Zustandsvektor --> Anfangszustand (aus ersten Messwerten)
  x = [PosMess(1);PosMess(2);PosMess(3);0;0;0]; 
  
  %Kovarianzmatrix des Schtzfehlers (Am Anfang Schtzung als ungenau
  %annehmen)
  P =  eye(6)*1000; 
  
  
  elseif(strcmp('CONST_ACCELERATION',Model))
      
  %Messmatrix
  H = [1 0 0 0 0 0 0 0 0;
       0 1 0 0 0 0 0 0 0;
       0 0 1 0 0 0 0 0 0];
  
  %Zustandsvektor --> Anfangszustand (aus ersten Messwerten)
  x = [PosMess(1);PosMess(2);PosMess(3);0;0;0;0;0;0];   
  
  %Kovarianzmatrix des Schtzfehlers (Am Anfang Schtzung als ungenau
  %annehmen)
  P =  eye(9)*1000; 
      
  end
  x_old = x;
  P_old = P;
end

%% Systemmatrix neu bestimmen

if(strcmp('CONST_VELOCITY',Model)) 
%Bewegungsmodell fr 'CONST_VELOCITY'     
A = [ 1 0 0 dt 0 0; 
      0 1 0 0 dt 0; 
      0 0 1 0 0 dt;
      0 0 0 1 0 0;
      0 0 0 0 1 0;
      0 0 0 0 0 1];
    
elseif(strcmp('CONST_ACCELERATION',Model))    
%Bewegungsmodell fr 'CONST_ACCELERATION'     
A = [ 1 0 0 dt 0 0 0.5*dt^2 0 0; 
      0 1 0 0 dt 0 0 0.5*dt^2 0; 
      0 0 1 0 0 dt 0 0 0.5*dt^2;
      0 0 0 1 0 0 dt 0 0;
      0 0 0 0 1 0 0 dt 0;
      0 0 0 0 0 1 0 0 dt;
      0 0 0 0 0 0 1 0 0;
      0 0 0 0 0 0 0 1 0;
      0 0 0 0 0 0 0 0 1];
    
end


%% 1. Vorhersage des Zustandsvektors und der Kovarianz
xp = A*x_old;            % Prdiktion des Zustandes
Pp = A*P_old*A' + Q;     % Prdiktion der Fehlerkovarianz

%Nur Vorhersage treffen, wenn Messwert vorhanden, sonst keine Korrektur
%vornehmen

if ~isnan(PosMess(1)) ||  ~isnan(PosMess(2)) ||  ~isnan(PosMess(3)) 
    
%% 2. Berechnung der Kalman Verstrkung K
    K = Pp*H'*inv(H*Pp*H' + R);
    
    
%% 3. Messen & Korrektur der Zustandsschtzung
    z = PosMess;
    x = xp + K*(z - H*xp);

    
%% 4. Korrektur der Kovarianzschtzung
    P = Pp - K*H*Pp;

else
    x = xp;
    P = Pp;
end


end