%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% Funktion        : Kalibrierung fr Roboter<-->Kinect Skalierung         %
%                                                                         %
%                                                                         %
% Eingabe         : rx: UR10 Konstante --> max Wert in x-Richtung         %
%                   ry: UR10 Konstante --> max Wert in y-Richtung         %
%                   rz: UR10 Konstante --> max Wert in z-Richtung         %
%                   offset: Verlngerung des Arms in x,y,z Richtung in m  %
%                   Kin2Rob: Transformationmatrix Kinect-->Roboter        %
%                   colorVid: Videoobjekt der Kinect (RGB-Kamera)         %
%                   depthVid: Videoobjekt der Kinect (Tiefensensor)       %
%                   Min_area: minimale Begrenzungslinie in x-Richtung     %
%                   Max_area: maximale Begrenzungslinie in x-Richtung     %
%                   Socket_conn: TCP/IP Verbindung zu UR10                %
%                                                                         %
%                                                                         %
%                                                                         %
% Ausgabe         : S: Skalierungsmatrix 3x3 mit [sx 0 0; 0 sy 0; 0 0 sz] %
%                   l1: kalibrierte Lnge des Oberarms                    %
%                   l2: kalibrierte Lnge des Unterarms                   %
%                                                                         %
%                                                                         %
% Autor           : Kevin Penner                                          %
%                                                                         %
% Implementation  : MATLAB R2016b                                         %
%                                                                         %
% Bemerkung       :                                                       %
%                                                                         %
% nderungsdatum  : 03.05.2017                                            %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [S, l1, l2] = KinectCalibration(rx, ry, rz, offset, Kin2Rob, colorVid, depthVid, Min_area, Max_area, Socket_conn)
    
%% Audiodateien laden
[Audio_up,Fs_up] = audioread('up.mp3');
[Audio_rightwards,Fs_rightwards] = audioread('rightwards.mp3');
[Audio_forwards,Fs_forwards] = audioread('forwards.mp3');
[Audio_start_calibration,Fs_start_calibration] = audioread('start_calibration.mp3');
[Audio_end_of_calibration,Fs_end_of_calibration] = audioread('end_of_calibration.mp3');


%% Speicher fr Kalibrierdaten
%Transformationsmatrix
S = zeros(3,3);
%Oberarm- (l1) und Unteramlnge (l2)
l1 = 0;
l2 = 0;

%% Einstellungen fr Roboter
%Roboterarmstellungen 
q_rightwards = [0,-pi/2,0,-pi/2,-pi/2,0];
q_up = [0,-pi,0,-pi/2,0,0];
q_forwards = [-pi/2,-pi,0,-pi/2,0,0];

%Roboter Einstellungen fr die Bewegung
%Beschleunigung in rad/s^2
accel_radss = 3;
%Geschwindigkeit in rad/s --> Max:130/s
vel_deg = 70;
vel_rads = deg2rad(vel_deg);
%Wird nicht bentigt!!!
time_robot = 0;
blend_radius = 0;


%% Kalibrierung
figure(1);
val = '0';
calib_state = 0;
calib_flag = 0;
index = 0;
calib_i = 0;

sound(Audio_start_calibration,Fs_start_calibration);
pause(2);
sound(Audio_rightwards,Fs_rightwards);
Befehl = command2string(q_rightwards,accel_radss,vel_rads,time_robot,blend_radius);
fprintf(Socket_conn, Befehl);

while calib_state < 3
    
   %Trigger aktivieren  
   trigger(depthVid);
   trigger(colorVid);
   %Bilder aufnehmen + Metadaten der Bilder
   [colorImage, ~, ~] = getdata(colorVid);
   [~, ~, depthMeta] = getdata(depthVid);
   %Skelettpunkte tracken und Anzahl der Trackingpunkte berechnen
   trackedBodies = find(depthMeta.IsBodyTracked);
   nBodies = length(trackedBodies);
   %Skelettkoordinaten berechnen
   colorJointIndices = depthMeta.ColorJointIndices(:, :, trackedBodies);
   
   %Bildausgabe
   imshow(colorImage);
   if calib_flag == 0 && calib_state == 0
      title('Armposition fr Robo.Z - linken Arm ber den Kopf heben, um Kalibrierung zu starten'); 
   elseif calib_flag == 0 && calib_state == 1
       title('Armposition fr Robo.X - linken Arm ber den Kopf heben, um Kalibrierung zu starten'); 
   elseif calib_flag == 0 && calib_state == 2
       title('Armposition fr Robo.Y - linken Arm ber den Kopf heben, um Kalibrierung zu starten');    
   elseif calib_flag == 1
       title('Kalibrierung erfolgt');
   end
   hold on;
   
   %Kalibrierung
   %Nur weiter rechnen, wenn ein Skelett getrackt wurde und nicht mehr als
   %2 Personen im Bild
   if sum(depthMeta.IsBodyTracked) > 0 && nBodies < 2
   
       %berprfen, ob linke Hand oben ist (ber dem Kopf)
       Wrist_Left = depthMeta.JointPositions(7,:,trackedBodies);
       Head = depthMeta.JointPositions(4,:,trackedBodies);
       if Wrist_Left(2) > Head(2)

           %Kalibrierung starten --> Flag setzen, solange linker Arm ber dem
           %Kopf
           calib_flag = 1;
           index = index + 1;

           %Rechte Schulter, Ellbogen und Armgelenk erkennen
           Shoulder_Right(index,:) = depthMeta.JointPositions(9,:,trackedBodies);
           Elbow_Right(index,:) = depthMeta.JointPositions(10,:,trackedBodies);
           Wrist_Right(index,:) = depthMeta.JointPositions(11,:,trackedBodies);

           %nderung der Achsen X/Y/Z --> In Roboterkoordinatensystem 
           Shoulder_Right(index,:) = Shoulder_Right(index,:)*Kin2Rob;
           Elbow_Right(index,:) = Elbow_Right(index,:)*Kin2Rob;
           Wrist_Right(index,:) = Wrist_Right(index,:)*Kin2Rob;

           %Neuer Nulllpunkt --> Schulter
           Elbow_Right(index,:) = Elbow_Right(index,:) - Shoulder_Right(index,:);
           Wrist_Right(index,:) = Wrist_Right(index,:) - Shoulder_Right(index,:);
           Shoulder_Right(index,:) = [0 0 0];


       elseif Wrist_Left(2) <= Head(2) && calib_flag == 1
            
            calib_flag = 0;
            calib_state = calib_state+1;
            calib_i(calib_state) = index;
            
            switch(calib_state)
                case 1
                    sound(Audio_up,Fs_up);
                    Befehl = command2string(q_up,accel_radss,vel_rads,time_robot,blend_radius);
                    fprintf(Socket_conn, Befehl);
                case 2
                    sound(Audio_forwards,Fs_forwards);
                    Befehl = command2string(q_forwards,accel_radss,vel_rads,time_robot,blend_radius);
                    fprintf(Socket_conn, Befehl);
                case 3
                    sound(Audio_end_of_calibration,Fs_end_of_calibration);
            end

       end
       
       %Ausgabe des rechten Arms
       for i = 6:7
         for body = 1:nBodies
            Shoulder_Img = colorJointIndices(9,1:2);
            Elbow_Img = colorJointIndices(10,1:2);
            Wrist_Img = colorJointIndices(11,1:2);
            ImageCoordinates(1,:) = Shoulder_Img;
            ImageCoordinates(2,:) = Elbow_Img; 
            ImageCoordinates(3,:) = Wrist_Img;  
            plot(ImageCoordinates(:,1),ImageCoordinates(:,2),'LineWidth',2,'Marker','o','MarkerSize',3,'MarkerEdgeColor','b','MarkerFaceColor','b','Color','r');
         end
       hold on;
       end
       %Ausgabe der Kopfbegrenzungslinie (y-Wert)
       [height,width] = size(colorImage);
       head_y = colorJointIndices(4,2);
       head_color(1,:) = [1 head_y];
       head_color(2,:) = [width head_y];
       line(head_color(:,1),head_color(:,2), 'LineWidth', 2, 'LineStyle', '-', 'Marker', '+', 'Color', 'b');
       hold on;
       %Ausgabe des linken Armpunktes
       wrist_left_color = colorJointIndices(7,:);
       plot(wrist_left_color(1),wrist_left_color(2),'Marker','*','Color','g');
       hold on;
       %Ausgabe der Begrenzungslinie
       area_min(1,:) = [Min_area 1];
       area_min(2,:) = [Min_area height];
       area_max(1,:) = [Max_area 1];
       area_max(2,:) = [Max_area height];
       center(1,:) = [width/6 1];
       center(2,:) = [width/6 height];
       line(area_min(:,1),area_min(:,2), 'LineWidth', 2, 'LineStyle', '-', 'Marker', '+', 'Color', 'm');
       hold on;
       line(area_max(:,1),area_max(:,2), 'LineWidth', 2, 'LineStyle', '-', 'Marker', '+', 'Color', 'm');
       hold on;
       line(center(:,1),center(:,2), 'LineWidth', 2, 'LineStyle', '-', 'Marker', '+', 'Color', 'm');
       hold on;
       
   end
   
   %Bereinigung 
   flushdata(colorVid);
   flushdata(depthVid);
   
   
    %Bei Bettigung der Esc-Taste --> Beenden
    val = double(get(figure(1), 'CurrentCharacter'));
    if(val==27)
        break; 
    end
   
end

if(val==27)
    close(1);
    return; 
end

%% Berechnung der Skaliesrungswerte
%Grtes Element finden aus: seitlich
[sz,i_sz] = max(Wrist_Right(1:calib_i(1),3));
Max_Wrist = Wrist_Right(i_sz,:);
Max_Elbow = Elbow_Right(i_sz,:);
%Armlngen berechnen
l1 = norm(Max_Elbow);
l2 = norm(Max_Wrist - Max_Elbow);

% %Lngen des Arms ndern
% for i=calib_i(1)+1:calib_i(3)
%     
%     Elbow_Right(index,:) = (l1/norm(Elbow_Right(index,:)))*Elbow_Right(index,:);
%     Wrist_Right(index,:) = Elbow_Right(index,:)+ (l2/norm(Wrist_Right(index,:)))*Wrist_Right(index,:);
%     
% end

%Grtes Element finden aus: nach oben
[sx,i_sx] = max(Wrist_Right(calib_i(1)+1:calib_i(2),1));

%Grtes Element finden aus: nach vorne
[sy,i_sy] = max(Wrist_Right(calib_i(2)+1:calib_i(3),2));

%Skalierungsfaktoren bestimmen 
sx = rx/(sx+offset);
sy = ry/(sy+offset);
sz = rz/(sz-offset);

%Transformationmatrix aufstellen
S = [sx 0 0; 0 sy 0; 0 0 sz];
    
close(1);
end